/* ************************************************************************
*   File: db.h                                          Part of CircleMUD *
*  Usage: header file for database handling                               *
*                                                                         *
*  All rights reserved.  See license.doc for complete information.        *
*                                                                         *
*  Copyright (C) 1993 by the Trustees of the Johns Hopkins University     *
*  CircleMUD is based on DikuMUD, Copyright (C) 1990, 1991.               *
************************************************************************ */
/* Archipelago changes by Alastair J. Neil Copyright (C) 1993, 94, 95, 96, 99 */
#ifndef DB_H
#define DB_H

/*define thread save for threaded saving undefine for normal */
#define THREAD_SAVE   

/* arbitrary constants used by index_boot() (must be unique) */
#define DB_BOOT_WLD	0
#define DB_BOOT_MOB	1
#define DB_BOOT_OBJ	2
#define DB_BOOT_ZON	3
#define DB_BOOT_SHP	4

/* names of various files and directories */

#define INDEX_FILE	"index"		/* index of world files		*/
#define MINDEX_FILE	"index.mini"	/* ... and for mini-mud-mode	*/
#define WLD_PREFIX	"world/wld"	/* room definitions		*/
#define MOB_PREFIX	"world/mob"	/* monster prototypes		*/
#define OBJ_PREFIX	"world/obj"	/* object prototypes		*/
#define ZON_PREFIX	"world/zon"	/* zon defs & command tables	*/
#define SHP_PREFIX	"world/shp"	/* shop definitions		*/

#define CREDITS_FILE	"text/credits"	/* for the 'credits' command	*/
#define NEWS_FILE	"text/news"	/* for the 'news' command	*/
#define MOTD_FILE	"text/motd"	/* messages of the day / mortal	*/
#define IMOTD_FILE	"text/imotd"	/* messages of the day / immort	*/
#define IDEA_FILE	"text/ideas"	/* for the 'idea'-command	*/
#define TYPO_FILE	"text/typos"	/*         'typo'		*/
#define BUG_FILE	"text/bugs"	/*         'bug'		*/
#define HELP_KWRD_FILE	"text/help_table"/* for HELP <keywrd>		*/
#define HELP_PAGE_FILE	"text/help"	/* for HELP <CR>		*/
#define INFO_FILE	"text/info"	/* for INFO			*/
#define WIZLIST_FILE	"text/wizlist"	/* for WIZLIST			*/
#define IMMLIST_FILE	"text/immlist"	/* for IMMLIST			*/
#define BACKGROUND_FILE	"text/background" /* for the background story	*/
#define POLICIES_FILE	"text/policies"	/* player policies/rules	*/
#define HANDBOOK_FILE	"text/handbook"	/* handbook for new immorts	*/
#define BUILDINGH_FILE  "text/building_h" /* building hints */

#define OBJ_LIMITS_FILE "misc/obj_limits" /* the objects limits data */
#define PLAYER_FILE2	"players-new"	/* the player database		*/
#define PLAYER_FILE	"misc/players"	/* the player database		*/
#define MESS_FILE	"misc/messages"	/* damage message		*/
#define SOCMESS_FILE	"misc/socials"	/* messgs for social acts	*/
#define MAIL_FILE	"misc/plrmail"	/* for the mudmail system	*/
#define BAN_FILE	"misc/badsites"	/* for the siteban system	*/
#define XNAME_FILE	"misc/xnames"	/* invalid name substrings	*/

/* public procedures in db1.c and db2.c */

int	count_hash_records(FILE *fl);
int     obj_is_in_limit_table(int obj_no);
void	boot_db(void);
void	char_to_store(struct char_data *ch, struct char_file_u *st);
void	store_to_char(struct char_file_u *st, struct char_data *ch);
#ifdef THREAD_SAVE
void    *thread_save(void *);
#endif

void   	save_rooms(int zone);
int   	write_rooms(FILE *room_f, int realzone, int vzone);
void   	save_objects(int zone);
int   	write_objects(FILE *obj_f, int realzone, int vzone);
void    save_zone(int zone);
int     write_zone(FILE *zone_f, int realzone, int vzone);
void   	save_mobiles(int zone);
int   	write_mobiles(FILE *mob_f, int realzone, int vzone);
void    save_shops(int zone);
int     write_shops(FILE *shop_f, int realzone, int vzone);

void	save_char(struct char_data *ch, sh_int load_room);

void	zone_update(void);
void	init_char(struct char_data *ch);
void	clear_char(struct char_data *ch);
void	clear_object(struct obj_data *obj);
void	reset_char(struct char_data *ch);
char	*fread_string(FILE *fl, char *error);

int	real_room(int virtual);
int	real_object(int virtual);
int	real_mobile(int virtual);

int	vnum_mobile(char *searchname, struct char_data *ch);
int	vnum_object(char *searchname, struct char_data *ch);
int	vnum_zone(char *searchname, struct char_data *ch);

void    write_db_index(int which);
void    copy_stringinfo(char **to, char **from);

int	load_char(char *name, struct char_file_u *char_element);
int	load_char_bynum(int player_i, struct char_file_u *char_element);
void	load_rooms(FILE *fl);
void	load_objects(FILE *obj_f);
void	load_mobiles(FILE *mob_f);

int     destroy_zreset(int zn, int no);

int	create_entry(char *name);
int	create_ol_entry(int obj_num);
int     create_zreset(int zn);
int     create_zone(int zone);
int     create_mob(int zone);
int     create_obj(int zone);
int     create_room(int zone);
int     create_shop(int zone);

void    make_dummy_mob(int i);
void    make_dummy_obj(int i);
void    make_dummy_room(int i);

void    clone_room(int from, int to);
void    clone_mob(int from, int to);

void    fwrite_string(FILE *wfile,char *wstr);
void	setup_dir(FILE *fl, int room, int dir, int mode);

void	free_char(struct char_data *ch);
void	free_obj(struct obj_data *obj);
void    free_room(struct room_data *room);
void    free_shop(struct shop_data *shop);
void    free_zone(struct zone_data *zone);
void    free_objstruct(struct obj_data *obj);
void    free_mob(struct char_data *mob);

void    copy_room(struct room_data *from, struct room_data *to);
void    copy_zone(struct zone_data *from, struct zone_data *to);
void    copy_shop(struct shop_data *from, struct shop_data *to);
void    copy_obj(struct obj_data *from, struct obj_data *to, bool copy_extra);
void    copy_mob(struct char_data *from, struct char_data *to);
void    copy_extras(struct extra_descr_data **target, struct extra_descr_data *exfrm );
void    save_all_object_limit_data(void);
void    load_limited(int obj_no);
void    extract_limited(int obj_no);
void    limited_to_char(int obj_no);
void    limited_from_char(int obj_no);
bool    is_maxed(int num, int mode);
void    save_an_object_limit_datum(int which);
void	index_boot(int mode);
void	old_index_boot(int mode);
void    load_obj_limits_data(void);
void	load_zones(FILE *fl);
void	boot_the_shops(FILE *shop_f, char *filename);
void	assign_mobiles(void);
void	assign_objects(void);
void	assign_rooms(void);
void	build_player_index(void);
int	is_empty(int zone_nr);
void	reset_zone(int zone);
int	file_to_string(char *name, char *buf);
int	file_to_string_alloc(char *name, char **buf);
void	check_start_rooms(void);
void	renum_world(void);
void	renum_objects(void);
void	renum_zone_table(void);
void	reset_time(void);
void    obj_zone_table(void);
void	clear_room(struct room_data *room);
void    check_items_in_rent(long vnum);
void    check_obj_limits_data(void);
bool    twinkie_mob(struct char_data *mob);
int     total_cp(struct obj_data *obj, int *max, int *number);
void    extract_twinkie(struct char_data *mob);
void    remove_items(struct obj_data **obj);


#define REAL 0
#define VIRTUAL 1

struct obj_data *read_object(int nr, int type, int mode);
struct char_data *read_mobile(int nr, int type);

ACMD(do_reboot);

/*structure for the object limits data */
struct obj_limit_type
{
    long obj_num;
    int no_stored;
};

struct obj_limit_list_type
{
    long obj_num;
    int no_stored;
    int no_loaded;
    int modified;
};

/* structure for the reset commands */
struct reset_com {
  char	command;   /* current command                      */
  
  bool if_flag;   /* if TRUE: exe only if preceding exe'd */
  int	arg1;       /*                                      */
  int	arg2;       /* Arguments to the command             */
  int	arg3;       /*                                      */
  int   percent;    /* percent chance of reset */
  int   num;        /*  number of times this reset has triggered */
  bool  twinkie; /* set for mobiles if they have too much good equipment */
   /* 
	*  Commands:              *
	*  'M': Read a mobile     *
	*  'O': Read an object    *
	*  'G': Give obj to mob   *
	*  'P': Put obj in obj    *
	*  'G': Obj to char       *
	*  'E': Obj to char equip *
	*  'D': Set state of door *
   */
};



/* zone definition structure. for the 'zone-table'   */
struct zone_data {
  char	*name;		    /* name of this zone                  */
  int	lifespan;           /* how long between resets (minutes)  */
  int	age;                /* current age of this zone (minutes) */
  int	top;                /* upper limit for rooms in this zone */
  char  open;
  
  int	reset_mode;         /* conditions for reset (see below)   */
  int	number;		    /* virtual number of this zone	  */
  struct reset_com *cmd;   /* command table for reset	          */
  struct zone_obj_list *list;
   /*
	*  Reset mode:                              *
	*  0: Don't reset, and don't update age.    *
	*  1: Reset if no PC's are located in zone. *
	*  2: Just reset.                           *
   */
};

/* struct for creating a sorted list of objects */

struct zone_obj_list {
  int vnum;
  struct obj_data *obj;
  struct zone_obj_list *next;
};

/* element in monster and object index-tables   */
struct index_data {
   int	virtual;    /* virtual number of this mob/obj           */
   int	number;     /* number of existing units of this mob/obj	*/
   SPECIAL(*func);  /* special procedure for this mob/obj       */
};




/* for queueing zones for update   */
struct reset_q_element {
   int	zone_to_reset;            /* ref to zone_data */
   struct reset_q_element *next;
};



/* structure for the update queue     */
struct reset_q_type {
   struct reset_q_element *head;
   struct reset_q_element *tail;
} reset_q;



struct player_index_element {
   char	*name;
   int id_num;
   bool playing;
};


struct help_index_element {
   char	*keyword;
   long	pos;
};

/* don't change these */
#define BAN_NOT 	0
#define BAN_NEW 	1
#define BAN_SELECT	2
#define BAN_ALL		3
#define BAN_SELF        4

#define BANNED_SITE_LENGTH    50
struct ban_list_element {
   char	site[BANNED_SITE_LENGTH+1];
   int	type;
   time_t	date;
   char	name[MAX_NAME_LENGTH+1];
   struct ban_list_element *next;
};


/* global buffering system */

#ifdef __DB_C__
char	buf[MAX_STRING_LENGTH];
char	buf1[MAX_STRING_LENGTH];
char	buf2[MAX_STRING_LENGTH];
char	arg[MAX_STRING_LENGTH];
#else
extern char	buf[MAX_STRING_LENGTH];
extern char	buf1[MAX_STRING_LENGTH];
extern char	buf2[MAX_STRING_LENGTH];
extern char	arg[MAX_STRING_LENGTH];
#endif

struct dns_entry {
    int ip[4];
};

/* For threaded saving of zone info. AJN 11/08/99 */

#define SAVE_UNDEFINED    0
#define SAVE_ROOMS        1
#define SAVE_OBJECTS      2
#define SAVE_MOBILES      3
#define SAVE_SHOPS        4
#define SAVE_ZONE         5

typedef struct threadsave{  
  int mode; /* SAVE_ROOMS|SAVE_OBJECTS|SAVE_MOBILES|SAVE_SHOPS|SAVE_ZONE */
  int zone;  /* real zone */
  int vzone; /* virtal zone */
  FILE *file;
} threadsave_t;

#endif
